/*
    Document   : dash_plugin_image_bootstrap3.js
    Created on : June 4, 2015, 00:04:26 PM
    Author     : Artur Barseghyan (artur.barseghyan@gmail.com)
    Description:
        Image plugin for django-dash.

        The following piece of code insures that images are opened using the
        bootstrap 3 modal.
*/
;
$(document).ready(function() {
    // Add dashboard plugin.
    //$('.pictonic .plugin-content-wrapper a').colorbox({
    //    'opacity': '0.5'
    //});
});
