__title__ = 'dash.contrib.plugins.dummy'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'default_app_config',
    'UID',
)

default_app_config = 'dash.contrib.plugins.dummy.apps.Config'

UID = 'dummy'
