{% load i18n %}
<div class="col-sm-10 col-md-10 col-lg-10  col-sm-offset-2 col-md-offset-2 col-lg-offset-2 main">
    <div class="page-header">
        <h1 class="content-title">{% blocktrans with plugin.name as plugin_name %}Add {{ plugin_name }} to Dashboard{% endblocktrans %}</h1>
    </div>

    <form method="post" action="{{ request.path }}" enctype="multipart/form-data" class="form-horizontal">
        {% csrf_token %}
        {% include layout.form_snippet_template_name %}
        <div class="form-group">
            <div class="col-sm-offset-2 col-sm-10">
                <button type="submit" class="btn btn-primary">{% trans "Add" %}</button>
            </div>
        </div>
        {% include "dash/snippets/plugin_help_text.html" %}
    </form>
</div>
