{% load i18n widget_tweaks dash_tags %}
                <div class="well">
                    {% blocktrans %}Fields marked with <span class="required-symbol">*</span> are required{% endblocktrans %}
                </div>

            {% if form.non_field_errors %}
                <div class="errors">
                    {{ form.non_field_errors.as_ul }}
                </div>
            {% endif %}
            {% for field in form %}

                {% if field.is_hidden %}
                    <div class="hidden-entry">
                        <div class="hidden-field">{{ field }}</div>
                    </div>
                {% else %}
                    <div class="form-group{% if field.errors %} error{% endif %}">
                        {% get_form_field_type field as form_field_type %}
                        {% if form_field_type.is_checkbox %}
                            <div class="col-sm-offset-2 col-sm-9">
                              <div class="checkbox">
                                <label>
                                    {{ field }}{{ field.label }}{{ field.errors }}
                                    <span class="helptext help-block">{{ field.help_text }}</span>
                                </label>
                              </div>
                            </div>
                        {% else %}
                            <label class="col-sm-2 control-label" for="{{ field.id_for_label }}">
                                {{ field.label }}
                                {% if field.field.required %}
                                <span class="required-field">*</span>
                                {% endif %}
                            </label>

                            <div class="col-sm-9">
                                {{ field|add_class:'form-control' }}{{ field.errors }}
                                {% if field.help_text %}
                                <span class="helptext help-block">{{ field.help_text }}</span>
                                {% endif %}
                            </div>
                        {% endif %}
                    </div>
                {% endif %}

            {% endfor %}
