__title__ = 'dash.contrib.plugins.rss_feed.defaults'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'DEFAULT_CACHE_FOR',
    'DEFAULT_MAX_FEED_ITEMS',
    'DEFAULT_SHOW_TITLE',
    'DEFAULT_TRUNCATE_AFTER',
)

DEFAULT_CACHE_FOR = 3600
DEFAULT_MAX_FEED_ITEMS = 6
DEFAULT_SHOW_TITLE = True
DEFAULT_TRUNCATE_AFTER = 35
