__title__ = 'dash.contrib.apps.public_dashboard'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2017 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('default_app_config',)

default_app_config = 'dash.contrib.apps.public_dashboard.apps.Config'
